from pygame import Rect, Surface
from pygame.locals import SRCALPHA
from math import fabs

from constants import *

from random import randint, choice as randchoice

class Shuriken:
    def __init__(self,coords,left,images,owner):
        self.images = images
        self.left = left
        self.owner = owner
        self.rect = Rect(0,0,16,16)
        self.rect.center = coords
        self.status = "Active"
        self.frame = 0
    def update(self,solids=None,targets=None):
        if self.status == 'Active':
            if self.left:
                self.rect.centerx -= 5
            else:
                self.rect.centerx += 5
            self.check_targets(targets)
            self.check_collides(solids)
            if self.status == 'Active':
                if self.left:
                    self.rect.centerx -= 5
                else:
                    self.rect.centerx += 5
                self.check_targets(targets)
            if self.status == 'Active':
                if self.left:
                    self.rect.centerx -= 5
                else:
                    self.rect.centerx += 5
                self.check_collides(solids)
                self.check_targets(targets)
        elif self.status == 'Stuck in Ninja':
                offset = [self.frame[3][X], self.frame[3][Y]]
                if not (self.frame[2].left and self.frame[5]):
                    offset[X] *= -1
                    self.left = not self.frame[2].left
                else:
                    self.left = self.frame[2].left
                self.rect.centerx = self.frame[2].rect.centerx + offset[X]
                self.rect.centery = self.frame[2].rect.centery + offset[Y]
        self.check_collides(solids)
        self.check_targets(targets)
        self.update_frame()
    
    def get_blit_info(self):
        draw_image = None
        if self.status == 'Active':
            draw_image = self.images[self.frame][self.left]
        elif self.status == "Stuck in Terrain":
            if self.frame[1] % (self.frame[0]*4) > 2:
                draw_image = self.images[self.frame[2]][self.left]
            else:
                draw_image = Surface((20,20),SRCALPHA,32)
        elif self.status == "Stuck in Ninja":
            if self.frame[1] % (self.frame[0]*4) > 2:
                draw_image = self.images[self.frame[6]][self.left]
            else:
                draw_image = Surface((20,20),SRCALPHA,32)
        draw_rect = draw_image.get_rect()
        draw_rect.center = self.rect.center
        return [draw_image, draw_rect]
    
    def check_collides(self, solids=None):
        if solids == None:
            solids = self.owner.level.terrain
        if self.status == 'Active':
            for object in solids:
                if self.rect.colliderect(object.rect) and self.status == 'Active':
                    self.status = "Stuck in Terrain"
                    self.frame = [10,0,self.frame]
    
    def check_targets(self,targets=None):
        if targets == None:
            if self.owner.is_player:
                targets = self.owner.level.enemies
            else:
                targets = [self.owner.level.player]
        for current_ninja in targets:
            if current_ninja.status != 'Dead':
                #If the ninja is dead, shurikens don't stick to him
                if self.rect.colliderect(current_ninja.rect):
                    if self.status == 'Stuck in Terrain':
                        current_ninja.take_damage(1,current_ninja.left)
                    elif self.status == 'Active':
                        current_ninja.take_damage(2,self.left)
                        self.status = "Stuck in Ninja"
                        offset = [self.rect.centerx-current_ninja.rect.centerx,self.rect.centery-current_ninja.rect.centery]
                        self.frame = [10,0,current_ninja,offset,self.left,current_ninja.left,self.frame]
    def update_frame(self):
        if self.status == 'Active':
            self.frame += 1
            while self.frame >= 6:
                self.frame -= 6
        else:
            self.frame[1] += 1
            if self.frame[1] >= (self.frame[0]*4):
                self.frame[1] = 0
                self.frame[0] -= 1
            if self.frame[0] <= 1:
                self.owner.shurikens.remove(self)
    
    
class ShurikenMaker:
    def __init__(self,images,owner):
        self.images = images
        self.owner = owner
    def make(self):
        self.owner.shurikens.append(Shuriken(self.get_throw_coords(),self.owner.left,self.images,self.owner))
        if self.owner.is_player:
            self.owner.shurikens_left -= 1
    def get_throw_coords(self):
        throw_x = self.owner.rect.right
        if self.owner.left:
            throw_x = self.owner.rect.left
        throw_y = self.owner.rect.centery
        return [throw_x,throw_y]
class Entity:
    def __init__(self,coords,size,image_set,level):
        self.rect = Rect((0,0),size)
        self.rect.center = coords
        self.images = image_set
        self.level = level
        self.collides = [False]*4
        self.frame = 0
        
        self.frame_timer = 0
        self.attack_timer = 0
        
        self.status = "Stand"
        
        self.left = False
        
        self.velocity = [0,0]
        
        self.walking = False
        
    def get_blit_info(self):
        draw_image = self.get_image()
        draw_rect = draw_image.get_rect()
        draw_rect.centerx = self.rect.centerx
        draw_rect.bottom = self.rect.bottom
        return [draw_image, draw_rect]
    
    def get_image(self):
        return self.images[self.get_status()][self.left]
    
    def get_status(self):
        return self.status
    
    def update(self,solids=None):
        self.velocity[Y] += 1
        self.enforce_speed_limits()
        self.move(solids)
        self.update_from_collides()
    
    def enforce_speed_limits(self):
        if self.velocity[X] > 25:
            self.velocity[X] = 25
        elif self.velocity[X] < -25:
            self.velocity[X] = -25
        if self.velocity[Y] > 25:
            self.velocity[Y] = 25
        elif self.velocity[Y] < -25:
            self.velocity[Y] = -25
    
    def update_from_collides(self):
        if self.collides[DOWN]:
            if self.velocity[Y] > 30:
                self.velocity[Y] = -15
            elif self.velocity[Y] > 0:
                self.velocity[Y] = 0
        if self.collides[UP]:
            if self.velocity[Y] < -30:
                self.velocity[Y] = 15
            elif self.velocity[Y] < 0:
                self.velocity[Y] = 0
        if self.collides[RIGHT]:
            if self.velocity[X] > 30:
                self.velocity[X] = -15
            elif self.velocity[X] > 0:
                self.velocity[X] = 0
        if self.collides[LEFT]:
            if self.velocity[X] < -30:
                self.velocity[X] = 15
            elif self.velocity[X] < 0:
                self.velocity[X] = 0
        
    def move(self,solids=None):
        self.collides = [False]*4
        
        if self.velocity[X] > 45:
            self.velocity[X] = 45
        elif self.velocity[X] < -45:
            self.velocity[X] = -45
        if self.velocity[Y] > 45:
            self.velocity[Y] = 45
        elif self.velocity[Y] < -45:
            self.velocity[Y] = -45
        
        if solids == None:
            solids = self.level.terrain
        
        self.rect.centerx += self.velocity[X]
        self.collide_and_respond(solids,self.velocity[X],0)
        
        self.rect.centery += self.velocity[Y]
        self.collide_and_respond(solids,0,self.velocity[Y])
    
    def collide_and_respond(self,solids,dx,dy):
        for object in solids:
            if self.rect.colliderect(object.rect):
                side = self.get_side(dx,dy)
                if side == DOWN:
                    self.collides[DOWN] = True
                    self.rect.bottom = object.rect.top
                elif side == UP:
                    self.collides[UP] = True
                    self.rect.top = object.rect.bottom
                elif side == LEFT:
                    self.collides[LEFT] = True
                    self.rect.left = object.rect.right
                elif side == RIGHT:
                    self.collides[RIGHT] = True
                    self.rect.right = object.rect.left
                if object.spikes:
                    self.respond_to_spikes()
    
    def respond_to_spikes(self):
        #This is to be overridden by derived classes
        pass
    
    def get_side(self,dx,dy):
        if fabs(dx) > fabs(dy):
            dy = 0
        else:
            dx = 0
        if dx > 0:
            return RIGHT
        elif dx < 0:
            return LEFT
        elif dy < 0:
            return UP
        elif dy > 0:
            return DOWN
        else:
            return -1
    
class Character(Entity):
    def __init__(self,coords,size,image_set,is_player,level):
        Entity.__init__(self,coords,size,image_set,level)
        
        if is_player:
            self.health = 25
        else:
            self.health = 5
        self.is_player = is_player
        
        self.shuriken_maker = ShurikenMaker(self.images['Shuriken'],self)
        self.shurikens = []
        self.shurikens_left = 15
        
        self.sword_areas = {True:Rect((0,0),size),False:Rect((0,0),size)}
        self.sword_areas[True].center = [self.rect.left,self.rect.centery]
        self.sword_areas[False].center = [self.rect.right,self.rect.centery]
        
        #This holds on to all of the ninjas who have hit this one with their
        # swords, so we don't take absurd damage
        self.damagers = []
        
        self.activated = False
    
    def respond_to_spikes(self):
        if self.status != 'Dead':
            left = down = False
            if self.velocity[X] > 0:
                left = True
            if self.velocity[Y] < 0:
                down = True
            self.take_damage(4,left,down)
    
    def get_blit_info(self):
        if self.status == 'Dead' and self.is_player == False:
            if self.frame_timer % self.frame >= 1:
                return Entity.get_blit_info(self)
            else:
                draw_image = Surface((self.rect.size),SRCALPHA,32)
                draw_rect = draw_image.get_rect()
                draw_rect.center = self.rect.center
                return [draw_image,draw_rect]
        else:
            return Entity.get_blit_info(self)
    
    def walk(self,go,left,run):
        if go == True and (self.status not in ('Dead','Hurt','Cheer')):
            self.walking = True
            self.left = left
            if run:
                top_speed = 15
            else:
                top_speed = 10
            
            if left:
                if self.velocity[X] > -(top_speed):
                    self.velocity[X] -= top_speed/5
                    if self.velocity[X] < -(top_speed):
                        self.velocity[X] = -(top_speed)
            else:
                if self.velocity[X] < top_speed:
                    self.velocity[X] += top_speed/5
                    if self.velocity[X] > top_speed:
                        self.velocity[X] = top_speed
        else:
            self.walking = False
    
    def update(self,solids=None,enemies=None,goal=None):
        self.frame_timer += 1
        if self.attack_timer > 0:
            self.attack_timer -= 1
        self.update_status()
        
        Entity.update(self,solids)
        self.check_damage(enemies)
        if self.is_player:
            self.check_goal(goal)
    
    def check_goal(self,goals=None):
        if goals == None:
            goals = self.level.goal
        
        for goal_post in goals:
            if self.rect.colliderect(goal_post.rect):
                self.status = 'Cheer'
    
    def move(self,solids=None):
        Entity.move(self,solids)
        self.sword_areas[True].center = [self.rect.left,self.rect.centery]
        self.sword_areas[False].center = [self.rect.right,self.rect.centery]
    
    def check_damage(self,enemies=None):
        if self.status != 'Dead':
            if enemies == None:
                if self.is_player:
                    enemies = self.level.enemies
                else:
                    enemies = [self.level.player]
            
            for bad_guy in enemies:
                if 'Sword' in bad_guy.status.split('-'):
                    if bad_guy not in self.damagers:
                        if self.rect.colliderect(bad_guy.sword_areas[bad_guy.left]):
                            self.damagers.append(bad_guy)
                            self.take_damage(3,bad_guy.left)
                else:
                    if bad_guy in self.damagers:
                        self.damagers.remove(bad_guy)
    
    def jump_start(self):
        if self.status not in ['Dead','Hurt','Cheer']:
            if self.status == 'Wall-Slide':
                self.velocity[Y] -= 15
                if self.velocity[Y] < -20:
                    self.velocity[Y] = -20
                if self.left:
                    self.velocity[X] += 10
                else:
                    self.velocity[X] -= 10
            elif self.collides[DOWN] == True and self.status != "Jump-Start":
                self.frame_timer = 0
                self.status = "Jump-Start"
    
    def jump(self):
        self.velocity[Y] -= 20
        if self.velocity[Y] < -25:
            self.velocity[Y] = -25
    def update_status(self):
        if self.health <= 0:
            self.status = 'Dead'
        if self.status == 'Cheer':
            return None
        elif self.status in ['Hurt','Dead']:
            if self.status == 'Hurt':
                while self.frame_timer >= 5:
                    self.frame_timer -= 5
                    self.frame += 1
                if self.frame >= 1:
                    self.frame = 0
                    if self.collides[DOWN]:
                        self.status = 'Stand'
                    else:
                        self.status = 'Fall-Air'
            elif self.status == 'Dead':
                while self.frame_timer >= 5:
                    self.frame_timer -= 5
                    self.frame -= 1
                if self.frame < 2:
                    self.frame = 2
        elif self.collides[DOWN]:
            if self.status == "Fall-Land":
                if self.frame_timer >= 5:
                    self.frame_timer = 0
                    self.status = "Stand"
            elif self.status == 'Jump-Start':
                if self.frame_timer >= 3:
                    self.frame_timer = 0
                    self.jump()
            else:
                    if 'Attack' in self.status.split('-'):
                        if 'Sword' in  self.status.split('-'):
                            while self.frame_timer >= 3:
                                self.frame_timer -= 3
                                self.frame += 1
                        else:
                            while self.frame_timer >= 2:
                                self.frame_timer -= 2
                                self.frame += 1
                        if self.frame >= 3:
                            self.frame_timer = 0
                            self.frame = 0
                            self.status = "Stand"
                    elif fabs(self.velocity[X]) > 12:
                        self.status = "Run"
                    elif fabs(self.velocity[X]) > 0:
                        self.status = "Walk"
                    elif self.velocity[X] == 0:
                        self.status = "Stand"
        elif self.status == 'Wall-Slide':
            if not any([self.collides[LEFT],self.collides[RIGHT]]):
                self.status = 'Fall-Air'
        else:
            if 'Attack' not in self.status.split('-'):
                if self.velocity[Y] < 0:
                    self.status = "Jump-Air"
                else:
                    self.status = "Fall-Air"
            else:
                if 'Sword' in  self.status.split('-'):
                    while self.frame_timer >= 3:
                        self.frame_timer -= 3
                        self.frame += 1
                else:
                    while self.frame_timer >= 2:
                        self.frame_timer -= 2
                        self.frame += 1
                if self.frame >= 3:
                    self.frame_timer = 0
                    self.frame = 0
                    self.status = "Stand"
        if self.status in ["Walk",'Run']:
            if self.status == 'Walk':
                while self.frame_timer >= 5:
                    self.frame_timer -= 5
                    self.frame += 1
            elif self.status == 'Run':
                while self.frame_timer >= 3:
                    self.frame_timer -= 3
                    self.frame += 1
            while self.frame >= 4:
                self.frame -= 4
        elif 'Attack' not in self.status.split('-') and self.status not in ['Dead','Hurt']:
            self.frame = 0
        while self.frame_timer >= 5:
            self.frame_timer -= 5
                
    def get_status(self):
        if self.status in ["Walk","Run"] or 'Attack' in self.status.split('-'):
            return self.status + '-' + str(self.frame)
        else:
            return self.status
    def update_from_collides(self):
        if self.status == 'Cheer':
            if self.collides[DOWN]:
                self.velocity[Y] = -15
                if self.velocity[X] >= 10:
                    self.velocity[X] -= 10
                elif self.velocity[X] < -10:
                    self.velocity[X] += 10
                elif self.velocity[X] != 0:
                    self.velocity[X] = 0
        elif self.status in ['Dead','Hurt']:
            if self.collides[DOWN]:
                if self.velocity[X] < 0:
                    self.velocity[X] += 1
                elif self.velocity[X] > 0:
                    self.velocity[X] -= 1
                if self.velocity[Y] > 10:
                    self.velocity[Y] *= -0.5
                elif self.velocity[Y] > 0:
                    self.velocity[Y] = 0
        else:
            if self.collides[DOWN]:
                if self.velocity[Y] > 3:
                    self.status = "Fall-Land"
                    self.frame_timer = 0
                    self.frame = 0
                if not self.walking:
                    if self.velocity[X] < -1:
                        self.velocity[X] += 1
                    elif self.velocity[X] < 0:
                        self.velocity[X] = 0
                    elif self.velocity[X] > 1:
                        self.velocity[X] -= 1
                    elif self.velocity[X] > 0:
                        self.velocity[X] = 0
                else:
                    if self.left:
                        if self.velocity[X] > 0:
                            self.velocity[X] -= 2
                        elif self.velocity[X] < -15:
                            self.velocity[X] += 1
                    else:
                        if self.velocity[X] < 0:
                            self.velocity[X] += 2
                        elif self.velocity[X] > 15:
                            self.velocity[X] -= 1
        if (self.collides[LEFT] or self.collides[RIGHT]) and not (self.collides[DOWN]) and self.health > 0:
            self.status = 'Wall-Slide'
            if self.velocity[Y] < -2:
                self.velocity[Y] += 1
            elif self.velocity[Y] > 0:
                self.velocity[Y] = 0
            if not self.is_player:
                if self.level.player.status == 'Cheer':
                    self.jump_start()
                    self.walk(True,self.left,True)
        Entity.update_from_collides(self)
    def attack(self, sword=True):
        if self.attack_timer <= 0 and (self.status not in ('Dead','Hurt','Cheer')):
            self.frame = 0
            self.attack_timer = 12
            if sword:
                if self.collides[DOWN]:
                    self.velocity[Y] -= 5
                    if self.left:
                        self.velocity[X] -= 5
                    else:
                        self.velocity[X] += 5
                    self.status = "Sword-Attack-Ground"
                else:
                    self.status = "Sword-Attack-Air"
            else:
                if self.shurikens_left > 0:
                    if self.collides[DOWN]:
                        self.status = "Shuriken-Attack-Ground"
                    else:
                        self.status = "Shuriken-Attack-Air"
                    self.shuriken_maker.make()
    def update_ai(self,player=None,solids=None):
        if player == None:
            player = self.level.player
        if player.status == 'Cheer':
            self.walk(True,(self.rect.centerx < player.rect.centerx),False)
            if len(self.shurikens) > 0:
                self.shurikens = []
        elif self.status not in ['Dead', 'Hurt']:
            if player.status != 'Dead':
                if fabs(self.rect.centery - player.rect.centery) >= 1000 or fabs(self.rect.centerx - player.rect.centerx) >= 1800:
                    if self.activated:
                        self.activated = False
                elif (fabs(self.rect.centerx-player.rect.centerx) <= 600 and fabs(self.rect.centery-player.rect.centery) <= 400) or self.activated:
                    if not self.activated:
                        self.activated = True
                    if self.status != 'Wall-Slide':
                        self.left = player.rect.centerx < self.rect.centerx
                    if fabs(self.rect.centerx-player.rect.centerx) < 45 and fabs(self.rect.centery-player.rect.centery) <= 30:
                        self.attack(True)
                    elif fabs(self.rect.centerx-player.rect.centerx) > 150 and fabs(self.rect.centery-player.rect.centery) <= 20:
                        self.walk(True,self.left,fabs(self.rect.centerx-player.rect.centerx) >= 50)
                        self.attack(False)
                    elif self.rect.centery - player.rect.centery > 45:
                        self.jump_start()
                        self.walk(True,self.left,fabs(self.rect.centerx-player.rect.centerx) >= 50)
                    else:
                        self.walk(True,self.left,fabs(self.rect.centerx-player.rect.centerx) >= 100)
            else:
                self.status = 'Cheer'
        if self.activated:
            self.update()
    def add_shurikens(self,amount):
        self.shurikens_left += amount
        if self.shurikens_left > 25:
            self.shurikens_left = 25
    def heal(self,amount):
        self.health += amount
        if self.health > 25 and self.is_player:
            self.health = 25
        elif self.health > 5 and not self.is_player:
            self.health = 5
    def take_damage(self,amount,left,go_down=False):
        self.health -= amount
        if self.health > 0:
            if left:
                self.velocity[X] -= amount*5
            else:
                self.velocity[X] += amount*5
            if go_down:
                self.velocity[Y] += amount*2
            else:
                self.velocity[Y] -= amount*2
            self.status = 'Hurt'
        else:
            self.health = 0
            self.status = 'Dead'
            self.frame = 15
            self.frame_timer = 0
        self.left = not left
    
class Item(Entity):
    def __init__(self,coords,size,image,level,kind,value):
        Entity.__init__(self,coords,size,{},level)
        self.kind = kind
        self.value = value
        self.value = (1,2,5,10,25)[self.value]
        self.image = image
    def update(self,player=None,solids=None):
        Entity.update(self,solids)
        self.check_for_player(player)
    def get_blit_info(self):
        draw_rect = self.image.get_rect()
        draw_rect.bottom = self.rect.bottom
        draw_rect.centerx = self.rect.centerx
        return [self.image,draw_rect]
    def check_for_player(self,player=None):
        if player == None:
            player = self.level.player
        if self.rect.colliderect(player.rect):
            self.give_to(player)
            self.status = 'Gone'
    def give_to(self,player):
        if self.kind == 'shurikens':
            player.add_shurikens(self.value)
        elif self.kind == 'health':
            player.heal(self.value)
