from pygame import Surface, Rect
from pygame.locals import SRCALPHA

from constants import *
from random import choice as random_choice

class TestTerrain:
    def __init__(self,coords,size):
        self.rect = Rect([0,0],size)
        self.rect.center = coords
        self.image = Surface(size,SRCALPHA,32)
        self.image.fill((0,125,0))
    def collides(self,other):
        return self.rect.colliderect(other.rect)
    def get_blit_info(self):
        draw_rect = Rect(self.rect.topleft,self.rect.size)
        return [self.image,draw_rect]
    
class Terrain:
    def __init__(self,type,coords,size,images):
        self.rect = Rect(coords,size)
        self.image = self.combine_images(images)
        self.spikes = (type == 'spikes')
    
    def combine_images(self,images):
        image = Surface(self.rect.size,SRCALPHA,32)
        for img,crds in images:
            image.blit(img,crds)
        return image
    
    def get_blit_info(self):
        draw_rect = self.image.get_rect()
        draw_rect.topleft = self.rect.topleft
        return [self.image,draw_rect]
    
class Goal:
    def __init__(self,coords,images,left):
        self.left = left
        self.rect = Rect(0,0,60,90)
        self.rect.center = coords
        self.images = images
    
    def get_blit_info(self):
        draw_rect = self.images[self.left].get_rect()
        draw_rect.center = self.rect.center
        return self.images[self.left],draw_rect
    
class Level:
    def __init__(self,size,player,terrain,goal,enemies,items):
        self.terrain = terrain
        self.goal = goal
        self.rect = Rect((0,0),size)
        self.enemies = enemies
        self.player = player
        self.items = items
        
        for ninja in self.enemies + [self.player]:
            ninja.level = self
        for ground_chunk in self.terrain:
            ground_chunk.level = self
        for goodie in self.items:
            goodie.level = self
        for goal_point in self.goal:
            goal_point.level = self
    
    def update_all(self):
        if self.player.status not in 'Dead':
            for blade in self.player.shurikens:
                blade.update()
                if blade.rect.right < -20 or blade.rect.left > (self.rect.right+20):
                    self.player.shurikens.remove(blade)
                elif blade.status == 'Stuck in Ninja':
                    if blade.frame[2].status == 'Dead':
                        self.player.shurikens.remove(blade)
        self.player.update()
        if self.player.rect.top > (self.rect.bottom + 50):
            self.player.status = 'Dead'
        elif self.player.rect.left < 0:
            self.player.rect.left = 0
            if self.player.velocity[X] < 0:
                self.player.velocity[X] = 0
        elif self.player.rect.right > self.rect.right:
            self.player.rect.right = self.rect.right
            if self.player.velocity[X] > 0:
                self.player.velocity[X] = 0
        
        for bad_guy in self.enemies:
            if bad_guy.status != 'Dead':
                for blade in bad_guy.shurikens:
                    blade.update()
                    if blade.rect.right < -20 or blade.rect.left > (self.rect.right+20):
                        bad_guy.shurikens.remove(blade)
                    elif blade.status == 'Stuck in Ninja':
                        if self.player.status == 'Dead':
                            bad_guy.shurikens.remove(blade)
                for other_bad_guy in self.enemies:
                    if bad_guy is not other_bad_guy and other_bad_guy.status != 'Dead':
                        if bad_guy.rect.colliderect(other_bad_guy.rect):
                            bad_guy.jump_start()
                            other_bad_guy.walk(True,random_choice([True,False]),random_choice([True,False]))
                            mod_velocity = random_choice([0,1])
                            if bad_guy.rect.centerx > other_bad_guy.rect.centerx:
                                mod_velocity *= -1
                            other_bad_guy.velocity[X] += mod_velocity
            bad_guy.update_ai()
            if bad_guy.rect.top > (self.rect.bottom + 50):
                #Set the enemy up for cleanup, he's dead
                bad_guy.status = 'Dead'
                bad_guy.frame = 2
            elif bad_guy.rect.left < 0:
                bad_guy.rect.left = 0
                if bad_guy.velocity[X] < 0:
                    bad_guy.velocity[X] = 0
            elif bad_guy.rect.right > self.rect.right:
                bad_guy.rect.right = self.rect.right
                if bad_guy.velocity[X] > 0:
                    bad_guy.velocity[X] = 0
            
            if bad_guy.status == 'Dead':
                if bad_guy.frame <= 3:
                    self.enemies.remove(bad_guy)
        
        for goodie in self.items:
            goodie.update()
            if goodie.status == 'Gone':
                self.items.remove(goodie)
