import pygame
from pygame.locals import *
from pygame import Surface, Rect

from factories import WorldFactory
from constants import *

from entity import Entity
from sys import argv

pygame.init()

class GameCamera:
    def __init__(self,size,level):
        self.level = level
        
        self.rect = Rect((0,0),size)
        self.area = self.level.rect
        if self.rect.width > self.area.width:
            self.rect.width = self.area.width
        if self.rect.height > self.area.height:
            self.rect.height = self.area.height
        
        self.anchor = self.level.player
        self.terrain = self.level.terrain
        self.ninjas = self.level.enemies + [self.level.player]
        self.items = self.level.items
        self.goal = self.level.goal
        self.background = []
        self.clouds = []
        #self.background = self.level.background
        #self.clouds = self.level.clouds
    
    def render(self,dest):
        result = Surface(self.rect.size,SRCALPHA,32)
        result.fill((5,100,200))
        self.rect.center = self.anchor.rect.center
        if self.rect.left < self.area.left:
            self.rect.left = self.area.left
        elif self.rect.right > self.area.right:
            self.rect.right = self.area.right
        if self.rect.bottom > self.area.bottom:
            self.rect.bottom = self.area.bottom
        elif self.rect.top < self.area.top:
            self.rect.top = self.area.top
        
        for prop in self.background:
            draw_image, draw_rect = prop.get_blit_info()
            if draw_rect.colliderect(self.rect):
                draw_rect.top -= self.rect.top
                draw_rect.left -= self.rect.left
                result.blit(draw_image,draw_rect.topleft)
        
        for goodie in self.items:
            if goodie.status != 'Gone':
                draw_image, draw_rect = goodie.get_blit_info()
                if draw_rect.colliderect(self.rect):
                    draw_rect.top -= self.rect.top
                    draw_rect.left -= self.rect.left
                    result.blit(draw_image, draw_rect.topleft)
        
        for ninja in self.ninjas:
            if ninja.status != 'Dead':
                for star in ninja.shurikens:
                    draw_image, draw_rect = star.get_blit_info()
                    if draw_rect.colliderect(self.rect):
                        draw_rect.top -= self.rect.top
                        draw_rect.left -= self.rect.left
                        result.blit(draw_image,draw_rect.topleft)
        
        for goal_post in self.goal:
            draw_image, draw_rect = goal_post.get_blit_info()
            if draw_rect.colliderect(self.rect):
                draw_rect.top -= self.rect.top
                draw_rect.left -= self.rect.left
                result.blit(draw_image,draw_rect.topleft)
        
        for ninja in self.ninjas:
            draw_image, draw_rect = ninja.get_blit_info()
            if draw_rect.colliderect(self.rect):
                draw_rect.top -= self.rect.top
                draw_rect.left -= self.rect.left
                result.blit(draw_image,draw_rect.topleft)
        
        for ground_chunk in self.terrain:
            draw_image, draw_rect = ground_chunk.get_blit_info()
            if draw_rect.colliderect(self.rect):
                draw_rect.top -= self.rect.top
                draw_rect.left -= self.rect.left
                result.blit(draw_image,draw_rect.topleft)
        
        for cloud in self.clouds:
            draw_image, draw_rect = cloud.get_blit_info()
            if draw_rect.colliderect(self.rect):
                draw_rect.top -= self.rect.top
                draw_rect.left -= self.rect.left
                result.blit(draw_image,draw_rect.topleft)
        
        dest.blit(result,[0,0])


def main_loop():
    ticker = pygame.time.Clock()
    screen = pygame.display.set_mode((600,400),SRCALPHA|HWSURFACE,32)
    
    pygame.display.set_caption("Rose Ninja -- Version 0.5 -- Pyweek 8 (Get Off My Lawn!)")
    
    factory = WorldFactory()
    mode = 'Start Load Level'
    
    paused = False
    
    current_level = None
    
    camera = None
    
    hud_font = pygame.font.SysFont('Sans,Arial',24,True,False)
    alert_font = pygame.font.SysFont('Sans,Arial',34,True,False)
    
    alert_loading = alert_font.render("Loading Level...",True,(150,150,150))
    alert_level_complete = alert_font.render("Level Complete!!",True,(150,150,150))
    alert_press_any_key = alert_font.render("Press Any Key!",True,(150,150,150))
    alert_you_have_died = alert_font.render("You Have Died!",True,(150,0,0))
    alert_welcome = alert_font.render("Rose Ninja -- V0.5",True,(240,255,0))
    
    alert_win = alert_font.render("You Have Won!",True,(255,255,0))
    alert_win_2 = alert_font.render("Thanks for Playing!",True,(255,255,0))
    
    loading_rect = alert_loading.get_rect()
    level_complete_rect = alert_level_complete.get_rect()
    press_any_key_rect = alert_press_any_key.get_rect()
    you_have_died_rect = alert_you_have_died.get_rect()
    welcome_rect = alert_welcome.get_rect()
    
    win_rect = alert_win.get_rect()
    win_2_rect = alert_win_2.get_rect()
    
    win_rect.bottom = win_2_rect.top = screen.get_rect().centery
    
    win_rect.centerx = win_2_rect.centerx = welcome_rect.centerx = \
    you_have_died_rect.centerx = loading_rect.centerx = level_complete_rect.centerx = \
    press_any_key_rect.centerx = screen.get_rect().centerx
    
    press_any_key_rect.bottom = loading_rect.bottom = screen.get_rect().bottom - 50
    welcome_rect.centery = you_have_died_rect.centery = level_complete_rect.centery = screen.get_rect().centery - 50
    
    score = 0
    current_level_number = 0
    max_level_number = len(factory.level_set)
    
    while True:
        ticker.tick(30)
        for e in pygame.event.get():
            if e.type == QUIT:
                exit()
            elif e.type == KEYDOWN:
                if e.key == K_q:
                    exit()
                elif current_level != None:
                    if e.key == K_a and not paused:
                        current_level.player.jump_start()
                    elif e.key == K_r and not paused:
                        current_level.player.status = 'Dead'
                        current_level.player.velocity[Y] -= 10
                    elif e.key == K_s and not paused:
                        current_level.player.attack(True)
                    elif e.key == K_w and not paused:
                        current_level.player.attack(False)
                    elif e.key == K_e:
                        paused = not paused
        keys_pressed = pygame.key.get_pressed()
        
        if not paused and current_level != None:
            current_level.player.walk(any([keys_pressed[K_LEFT],keys_pressed[K_RIGHT]]), keys_pressed[K_LEFT], keys_pressed[K_d])
            current_level.update_all()
        
        if camera != None:
            camera.render(screen)
        
        if mode == 'Game' and current_level != None:
            if current_level.player.status == 'Cheer':
                current_level_number += 1
                if current_level_number < max_level_number:
                    mode = 'Load Level'
                else:
                    mode = 'Player Has Won'
            elif current_level.player.status == 'Dead':
                mode = 'Die Load Level'
            else:
                hud_health = hud_font.render(("Health: " + str(current_level.player.health)),True,(200,0,0))
                hud_shurikens = hud_font.render(("Shurikens: " + str(current_level.player.shurikens_left)),True,(150,150,150))
                
                screen_rect = screen.get_rect()
                health_rect = hud_health.get_rect()
                shurikens_rect = hud_shurikens.get_rect()
                
                health_rect.topleft = screen_rect.topleft
                health_rect.top += 20
                health_rect.left += 20
                
                shurikens_rect.topleft = health_rect.bottomleft
                shurikens_rect.top += 5
                shurikens_rect.left += 20
                
                screen.blit(hud_health,health_rect.topleft)
                screen.blit(hud_shurikens,shurikens_rect.topleft)
        elif mode == 'Player Has Won':
            screen.blit(alert_win,win_rect.topleft)
            screen.blit(alert_win_2,win_2_rect.topleft)
            if pygame.key.get_pressed()[K_SPACE]:
                exit()
        elif mode == 'Load Level':
            screen.blit(alert_level_complete,level_complete_rect.topleft)
            screen.blit(alert_loading,loading_rect.topleft)
            pygame.display.update()
            next_level = factory.load_level(current_level_number)
            mode = 'Next Level Ready'
        elif mode == 'Next Level Ready':
            screen.blit(alert_level_complete,level_complete_rect.topleft)
            screen.blit(alert_press_any_key,press_any_key_rect.topleft)
            if any(pygame.key.get_pressed()):
                mode = 'Start Level'
        elif mode == 'Start Level':
            current_level = next_level
            camera = GameCamera(screen.get_rect().size,current_level)
            mode = 'Game'
        elif mode == 'Die Load Level':
            screen.blit(alert_you_have_died,you_have_died_rect.topleft)
            screen.blit(alert_loading,loading_rect.topleft)
            pygame.display.update()
            next_level = factory.load_level(current_level_number)
            mode = 'Retry Ready'
        elif mode == 'Start Load Level':
            screen.fill((5,100,200))
            screen.blit(alert_welcome,welcome_rect.topleft)
            screen.blit(alert_loading,loading_rect.topleft)
            pygame.display.update()
            next_level = factory.load_level(current_level_number)
            mode = 'Start Ready'
        elif mode == 'Start Ready':
            screen.fill((5,100,200))
            screen.blit(alert_welcome,welcome_rect.topleft)
            screen.blit(alert_press_any_key,press_any_key_rect.topleft)
            if any(pygame.key.get_pressed()):
                mode = 'Start Level'
        elif mode == 'Retry Ready':
            screen.blit(alert_you_have_died,you_have_died_rect.topleft)
            screen.blit(alert_press_any_key,press_any_key_rect.topleft)
            if any(pygame.key.get_pressed()):
                mode = 'Start Level'
        pygame.display.flip()
        
main_loop()
